// Compile: csc /r:System.Numerics.dll factorial2.cs

using System;
using System.Numerics;

public class FactorialTest2
{
	private delegate BigInteger Factorial(BigInteger n);
	
	public static void Main(string[]  args)
	{
		System.Console.WriteLine("Factorial recursive and iterative revisited");
		System.Console.WriteLine("-------------------------------------------" + System.Environment.NewLine);
		
		for(int i = 1; i < 5; i++)
		{
			System.Console.WriteLine(string.Format("{0}!", (int)Math.Pow(10, i)));
			
			RunFactorial(i, FactorialRecursive, "Recursive");
			RunFactorial(i, FactorialIterative, "Iterative");
			System.Console.WriteLine();
		}
	}
	
	private static void RunFactorial(int i, Factorial fn, string name)
	{
		DateTime start;
		DateTime stop;
		TimeSpan diff;
		
		start = DateTime.Now;
		
		for(BigInteger j = 0; j < 100000; j++)
		{
			fn((BigInteger)Math.Pow(10, i));
		}
		
		stop = DateTime.Now;
		
		diff = stop - start;
		System.Console.WriteLine(string.Format("{0}: {1}", name, diff));
	}
	
	private static BigInteger FactorialIterative(BigInteger n)
	{
		BigInteger localFactorial = 1;
		while(n > 1)
		{
			localFactorial *= n;
			n--;
		}
		
		return localFactorial;
	}
	
	private static BigInteger FactorialRecursive(BigInteger n)
	{
		if(n < 1) return 1;
		return n * FactorialRecursive(n -1);
	}
}
